package liferay.intranet.controller;


import java.io.File;

import java.io.IOException;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;

import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import org.springframework.web.portlet.bind.annotation.ActionMapping;
import org.springframework.web.portlet.bind.annotation.RenderMapping;

import com.liferay.portal.service.ServiceContextThreadLocal;
import javax.portlet.PortletContext;

import liferay.intranet.domain.App;
import liferay.intranet.domain.Group;
import liferay.intranet.service.AppService;


@Controller("appController")//controlador gestionat per Spring el qual crea el bean associat al fitxer de configuraci
@RequestMapping("VIEW")//el controlador s'encarrega de gestionar la vista del portlet
public class AppController  {


	@Autowired //s'encarrega de cercar un bean de la classe corresponent
	private AppService appService;
	
	@RenderMapping//indica a Spring que te que cercar un .jsp dins de la carpeta establerta amb aquest nom, per mostrar-lo a la vista del portlet
	public String listApps(Map<String, Object> map) throws com.liferay.portal.kernel.exception.PortalException, com.liferay.portal.kernel.exception.SystemException {
		String jsp;
		com.liferay.portal.service.ServiceContext serviceContext = ServiceContextThreadLocal.getServiceContext();
		
		Integer idUser = (int) (long)  serviceContext.getUserId();
		boolean admin = appService.isAdmin(idUser);
		
		//es comproba si l'usuari es administrador per mostrar una vista o una altra
		if (admin) {
			//carreguem el llistat de totes les aplicacions disponibles
			map.put("app", new App());
			map.put("appList", appService.listApps());
			
			//carreguem els grups a la taula per tal de poder-los mostrar al combobox del formulari
			Map referenceData = new HashMap();
			 
			Map<Integer,String> listGroups = new LinkedHashMap<Integer,String>();
			
			Iterator<Group> it = appService.listGroups().iterator();
			while (it.hasNext()){
			
				Group g = it.next();
				listGroups.put(g.getUserGroupId(), g.getName());
			}
			
			map.put("groupList", listGroups);				
			jsp = "app";
		}
		else{		
			//guardem a appList totes les aplicacions disponibles per a l'usuari registrat
			map.put("appList", appService.displayApps(idUser));
			jsp = "appuser";
		}
		return jsp;
	}
		
		
	
		@ActionMapping(params = "action=add")//rep les peticions dels usuaris, executen la lgica associada i estableixen quin mtode Render ser el seguent en ejecutar-se
		public void addApp(ActionRequest actionRequest,ActionResponse actionResponse, Model model, @ModelAttribute("app") App app,  BindingResult result) throws IOException,
				PortletException {
	   
			CommonsMultipartFile p = app.getIcon();
	
			// si hi ha icono carreguem l'icono escollit per l'usuari, sino n'introduim un per defecte
			if (p.getOriginalFilename()!=""){
		
				PortletContext context = actionRequest.getPortletSession().getPortletContext();
			
				String relativeWebPath = "/icons/";
				
				String realPath = context.getRealPath(relativeWebPath);
				
	
				String filePath = realPath + "/" + p.getOriginalFilename();
				
				
				System.out.println(filePath);
				
				File dest = new File(filePath);
					
				p.transferTo(dest);  
				
			
				app.setNomIcon(p.getOriginalFilename());
			
			}
			else {
				app.setNomIcon("defaulticon.jpg");
			}
			app.setIcon(null);
			appService.addApp(app);
			
			
		}
		
		//Elimina una aplicaci passant-li el @requestParam appId
		@ActionMapping(params = "action=delete")
		public void deleteApp(@RequestParam("appId") Integer appId,ActionRequest actionRequest,ActionResponse actionResponse, Model model) throws IOException,
				PortletException {
			appService.removeApp(appId);	
		}
		
		//Realitza una consulta a la base de dades per recuperar l'aplicaci conicident amb el @requestParam appId
		@ActionMapping(params = "action=consult")
		public void consultApp(@RequestParam("appId") Integer appId,ActionRequest actionRequest,ActionResponse actionResponse,  Model model) throws IOException,
				PortletException {
			
			//carregeum l'objecte app
			App app = appService.consultApp(appId);
			actionRequest.setAttribute("edit", "edit");
			actionRequest.setAttribute("consultApp", app);
			//guardem els grups disponibles per a poder-los utilitzar a la modificaci de l'apliacio
			List<Group> groupList  = appService.listGroups();
			actionRequest.setAttribute("groups", groupList);

	
		}
		
		//s'actualitza les dades de l'apliaci no li passem el @ModelAttribute("consultapp")
		@ActionMapping(params = "action=update")//rep les peticions dels usuaris, ejecuten la lgica asociada i estableixen quin mtode Render ser el seguent en ejecutar-se
		public void updateApp(ActionRequest actionRequest,ActionResponse actionResponse, Model model,@ModelAttribute("consultapp") App consultapp, BindingResult result) throws IOException,
				PortletException {
			
			
			CommonsMultipartFile p = consultapp.getIcon();
			//es comproba si la imatge esta buida
			if (p.getOriginalFilename()!=""){
				
				PortletContext context = actionRequest.getPortletSession().getPortletContext();
			
				String relativeWebPath = "/icons/";
				
				String realPath = context.getRealPath(relativeWebPath);
				
	
				String filePath = realPath + "/" + p.getOriginalFilename();
				
				
				System.out.println(filePath);
				
				File dest = new File(filePath);
					
				p.transferTo(dest);  
				
				
				consultapp.setIcon(null);
				consultapp.setNomIcon(p.getOriginalFilename());
			
			}
			else {
				consultapp.setNomIcon("defaulticon.jpg");
			}
			
			appService.editApp(consultapp);	
			
		}

	}



